#!/usr/bin/python
# -*- coding: utf8 -*-

import logging
import os
import sys
import pty
import time
import datetime
import math
import struct
import socket
import select

import PyIndi

USE_SERIAL = True
USE_TCP = True
TCP_LISTEN_PORT = 8091
DEVICE_PORT = "/tmp/indi-synscan"
INDI_SERVER_HOST = "localhost"
INDI_SERVER_PORT = 7624
TELESCOPE_DEVICE = "EQMod Mount"
TELESCOPE_SIMULATION = True
TELESCOPE_SIMPROP = "SIMULATION"


class IndiClient(PyIndi.BaseClient):
    global logger

    def __init__(self):
        super(IndiClient, self).__init__()
        self.isconnected = False

    def newMessage(self, d, m):
        logger.info("Message for " + d.getDeviceName() + ":" + d.messageQueue(m))

    def serverConnected(self):
        self.isconnected = True
        logger.info(
            "Server connected (" + self.getHost() + ":" + str(self.getPort()) + ")"
        )

    def serverDisconnected(self, code):
        self.isconnected = False
        logger.info(
            "Server disconnected (exit code = "
            + str(code)
            + ","
            + str(self.getHost())
            + ":"
            + str(self.getPort())
            + ")"
        )


numberFailures = set()
switchFailures = set()
textFailures = set()


def getNumberWithRetry(prop, ntry=5, delay=0.2):
    while ntry > 0:
        p = device.getNumber(prop)
        if p.isValid():
            numberFailures.discard(prop)
            return p
        if prop in numberFailures:
            return None
        logger.info("Unable to get number property " + prop + ", retrying")
        ntry -= 1
        time.sleep(delay)
    logger.info("Unable to get number property " + prop + ", marking as failed")
    numberFailures.add(prop)
    return None


def getSwitchWithRetry(prop, ntry=5, delay=0.2):
    while ntry > 0:
        p = device.getSwitch(prop)
        if p.isValid():
            switchFailures.discard(prop)
            return p
        if prop in switchFailures:
            return None
        logger.info("Unable to get switch property " + prop + ", retrying")
        ntry -= 1
        time.sleep(delay)
    logger.info("Unable to get switch property " + prop + ", marking as failed")
    switchFailures.add(prop)
    return None


def getTextWithRetry(prop, ntry=5, delay=0.2):
    while ntry > 0:
        p = device.getText(prop)
        if p.isValid():
            textFailures.discard(prop)
            return p
        if prop in textFailures:
            return None
        logger.info("Unable to get text property " + prop + ", retrying")
        ntry -= 1
        time.sleep(delay)
    logger.info("Unable to get text property " + prop + ", marking as failed")
    textFailures.add(prop)
    return None


def process_command(buf, indiclient, logger):
    global device
    # Default error
    reply_error = b"#"
    i = 0
    reply = b""
    while i < len(buf):
        cmd = buf[i]
        i += 1
        # Jump
        if cmd == ord(b"\x00"):
            pass
        elif cmd == ord("#"):
            reply += b"#"
        # Echo
        elif cmd == ord("K"):
            if not (indiclient.isconnected):
                reply += reply_error
            else:
                reply += buf[i : i + 1] + b"#"
            i += 1
        elif not (device) or not (indiclient.isconnected) or not (device.isConnected()):
            logger.info("Lost device " + TELESCOPE_DEVICE + ": cannot process command")
            return reply + reply_error
        # Alignment complete ?
        elif cmd == ord("J"):
            reply += b"\x01" + b"#"
        # Get Ra/Dec
        elif cmd in [ord("e"), ord("E")]:
            p = getNumberWithRetry("EQUATORIAL_EOD_COORD")
            if p is None:
                reply += reply_error
                continue
            radeg = (p[0].getValue() * 360.0) / 24.0
            decdeg = p[1].getValue()
            if decdeg < 0.0:
                decdeg = 360.0 + decdeg
            rahex = hex(int((radeg * 2**24) / 360.0))[2:].zfill(6).upper() + "00"
            dechex = hex(int((decdeg * 2**24) / 360.0))[2:].zfill(6).upper() + "00"
            if sys.version_info >= (3,):
                rahex = bytes(rahex, "ascii")
                dechex = bytes(dechex, "ascii")
            if cmd == ord("e"):
                reply += rahex + b"," + dechex + b"#"
            else:
                reply += rahex[0:4] + b"," + dechex[0:4] + b"#"
        # Get time
        elif cmd == ord("h"):
            p = getTextWithRetry("TIME_UTC")
            if p is None:
                reply += reply_error
                continue
            utc8601 = p[0].getText()
            if p[1].getText():
                offset = int(p[1].getText())
            else:
                offset = 0
            # could use dateutil.parser.parse
            utc = datetime.datetime.strptime(utc8601, "%Y-%m-%dT%H:%M:%S")
            utc = utc + datetime.timedelta(0, 0, 0, 0, 0, offset)
            if offset < 0:
                offset = 256 - offset
            reply += (
                struct.pack(
                    "BBBBBBBB",
                    utc.hour,
                    utc.minute,
                    utc.second,
                    utc.month,
                    utc.day,
                    utc.year - 2000,
                    offset,
                    0x00,
                )
                + b"#"
            )
        # Get Model
        elif cmd == ord("m"):
            p = getTextWithRetry("MOUNTINFORMATION")
            if p is None:
                m = b"!"
            else:
                skywatcher_models = {
                    "EQ6": b"\x00",
                    "HEQ5": b"\x01",
                    "EQ5": b"\x02",
                    "EQ3": b"\x03",
                    "EQ8": b"\x04",
                    "AZ-EQ6": b"\x05",
                    "AZ-EQ5": b"\x06",
                }
                if p[0].getText() in skywatcher_models:
                    m = skywatcher_models[p[0].getText()]
                else:
                    m = b"\x00"
            reply += m + b"#"
        # Get Location
        elif cmd == ord("w"):
            p = getNumberWithRetry("GEOGRAPHIC_COORD")
            if p is None:
                reply += reply_error
                continue
            latdeg = p[0].getValue()
            longdeg = p[1].getValue()
            elev = p[2].getValue()
            latd = b"\x00"
            if latdeg < 0.0:
                latd = b"\x01"
                latdeg = -(latdeg)
            latfrac, lata = math.modf(latdeg)
            latfrac, latb = math.modf(latfrac * 60)
            latfrac, latc = math.modf(latfrac * 60)
            longh = b"\x00"
            if longdeg > 180.0:
                longdeg -= 360.0
            if longdeg < 0.0:
                longh = b"\x01"
                longdeg = -(longdeg)
            longfrac, longe = math.modf(longdeg)
            longfrac, longf = math.modf(longfrac * 60)
            longfrac, longg = math.modf(longfrac * 60)
            reply += (
                struct.pack("BBB", int(lata), int(latb), int(latc))
                + latd
                + struct.pack("BBB", int(longe), int(longf), int(longg))
                + longh
                + b"#"
            )
        # Get Version
        elif cmd == ord("V"):
            # reply += b"21#"
            # reply += b"\x04\x0E#"
            # reply += b"\x03\x03#"
            # nex skywatcher ?
            # reply += b"042508#"
            reply += b"032507"
            # celestron / old skywatcher ?
            # reply += b"\x04\x25\x07"
            # reply += b"\x04\x25\x07#" normally with a # but this corrupts the indi-synscan driver
        # Set time
        elif cmd == ord("H"):
            [h, m, s, mth, d, y, offset, dst] = buf[i : i + 8]
            i += 8
            y += 2000
            if offset < 0:
                offset = 256 - offset
            lt = datetime.datetime(y, mth, d, h, m, s, 0, None)
            utc = lt - datetime.timedelta(0, 0, 0, 0, 0, offset)
            logger.info("Setting time to " + utc.isoformat() + " " + str(offset))
            p = getTextWithRetry("TIME_UTC")
            if p is None:
                reply += reply_error
                continue
            p[0].setText(utc.isoformat())
            p[1].setText(str(offset))
            indiclient.sendNewProperty(p)
            reply += b"#"
        # Set Location
        elif cmd == ord("W"):
            data = buf[i : i + 8]
            i += 8
            lat = data[0] + (data[1] / 60) + (data[2] / 3600)
            if data[3] == 1:
                lat = -lat
            long = data[4] + (data[5] / 60) + (data[6] / 3600)
            if data[7] == 1:
                long = 360.0 - long
            p = getNumberWithRetry("GEOGRAPHIC_COORD")
            if p is None:
                reply += reply_error
                continue
            p[0].setValue(lat)
            p[1].setValue(long)
            indiclient.sendNewProperty(p)
            reply += b"#"
        # Goto/Sync
        elif cmd in [ord("r"), ord("R"), ord("s"), ord("S")]:
            ingoto = cmd in [ord("r"), ord("R")]
            if cmd in [ord("r"), ord("s")]:
                rahour = (int(buf[i : i + 8], 16) * 24.0) / (2**32)
                decdeg = (int(buf[i + 9 : i + 17], 16) * 360.0) / (2**32)
                i += 17
            else:
                rahour = (int(buf[i : i + 4], 16) * 24.0) / (2**16)
                decdeg = (int(buf[i + 5 : i + 9], 16) * 360.0) / (2**16)
                i += 9
            if decdeg >= 270.0:  # I don't check for 90.0 < values < 270.0
                decdeg = decdeg - 360.0
            p = getNumberWithRetry("EQUATORIAL_EOD_COORD")
            if p is None:
                reply += reply_error
                continue
            p[0].setValue(rahour)
            p[1].setValue(decdeg)
            pcs = getSwitchWithRetry("ON_COORD_SET")
            if pcs is None:
                reply += reply_error
                continue
            if ingoto:
                pcs.reset()
                pcs[0].setState(PyIndi.ISS_ON)
                logger.info("Goto " + str(rahour) + ", " + str(decdeg))
            else:
                pcs.reset()
                pcs[2].setState(PyIndi.ISS_ON)
                logger.info("Sync " + str(rahour) + ", " + str(decdeg))
            indiclient.sendNewProperty(pcs)
            indiclient.sendNewProperty(p)
            reply += b"#"
        # in goto ?
        elif cmd == ord("L"):
            p = getNumberWithRetry("EQUATORIAL_EOD_COORD")
            if p is None:
                reply += reply_error
                continue
            if p.getState() == PyIndi.IPS_BUSY:
                reply += b"1#"
            else:
                reply += b"0#"
        # abort goto
        elif cmd == ord("M"):
            p = getNumberWithRetry("EQUATORIAL_EOD_COORD")
            if p is None:
                reply += reply_error
                continue
            if p.getState() == PyIndi.IPS_BUSY:
                p = getSwitchWithRetry("TELESCOPE_ABORT_MOTION")
                if p is None:
                    reply += reply_error
                    continue
                p[0].setState(PyIndi.ISS_ON)
                indiclient.sendNewProperty(p)
            reply += b"#"
        # MoveWE/MoveNS
        elif cmd == ord("P"):
            data = buf[i : i + 7]
            i += 7
            if data[0] != 2:  # variable rate not supported
                reply += reply_error
                continue
            if data[1] == 16:
                pmotionname = "TELESCOPE_MOTION_WE"
            else:  # should be 17
                pmotionname = "TELESCOPE_MOTION_NS"
            pmotion = getSwitchWithRetry(pmotionname)
            if pmotion is None:
                reply += reply_error
                continue
            rate = data[3]
            if rate == 0:  # stop
                pmotion.reset()
                indiclient.sendNewProperty(pmotion)
            else:
                prate = getSwitchWithRetry("TELESCOPE_SLEW_RATE")
                if prate is None or len(prate) < 1:  # no slew rate
                    reply += reply_error
                    continue
                prateswitches = {
                    "SLEW_GUIDE": None,
                    "SLEW_CENTERING": None,
                    "SLEW_FIND": None,
                    "SLEW_MAX": None,
                }
                for p in prate:
                    p.setState(PyIndi.ISS_OFF)
                    if p.getName() in prateswitches:
                        prateswitches[p.getName()] = p
                prateset = prate[len(prate) - 1]
                if rate == 1 and prateswitches["SLEW_GUIDE"]:
                    prateset = prateswitches["SLEW_GUIDE"]
                if 2 <= rate <= 4 and prateswitches["SLEW_CENTERING"]:
                    prateset = prateswitches["SLEW_CENTERING"]
                if 5 <= rate <= 7 and prateswitches["SLEW_FIND"]:
                    prateset = prateswitches["SLEW_FIND"]
                if 8 <= rate <= 9 and prateswitches["SLEW_MAX"]:
                    prateset = prateswitches["SLEW_MAX"]
                prateset.setState(PyIndi.ISS_ON)
                indiclient.sendNewProperty(prate)
                movedir = data[2]
                if movedir == 36:  # positive move i.e. West/North
                    pmotion[0].setState(PyIndi.ISS_ON)
                    pmotion[1].setState(PyIndi.ISS_OFF)
                else:  # should be 37 negative move i.e. East/South
                    pmotion[0].setState(PyIndi.ISS_OFF)
                    pmotion[1].setState(PyIndi.ISS_ON)
                indiclient.sendNewProperty(pmotion)
            reply += b"#"
        # Pierside
        elif cmd == ord("p"):
            p = getSwitchWithRetry("TELESCOPE_PIER_SIDE")
            if p is None:
                reply += reply_error
                continue
            if p[0].getState() == PyIndi.ISS_ON:  # PIER_EAST
                reply += b"E#"
            else:
                reply += b"W#"
        # Get Tracking
        elif cmd == ord("t"):
            p = getSwitchWithRetry("TELESCOPE_TRACK_RATE")
            if p is None:
                reply += reply_error
                continue
            mode = b"0"
            if any(p[n].getState() == PyIndi.ISS_ON for n in range(4)):
                mode = b"2"
            reply += mode + b"#"
        # Set Tracking
        elif cmd == ord("T"):
            mode = buf[i]
            i += 1
            p = getSwitchWithRetry("TELESCOPE_TRACK_RATE")
            if p is None:
                reply += reply_error
                continue
            if mode in [ord("2"), ord("3")]:  # EQ/PEC tracking (no Alt/Az)
                if p[0].getState() == PyIndi.ISS_OFF:
                    p.reset()
                    p[0].setState(ON)
                    indiclient.sendNewProperty(p)
            else:
                if any(p[n].getState() == PyIndi.ISS_ON for n in range(4)):
                    p.reset()
                    indiclient.sendNewProperty(p)
            reply += b"#"
        else:  # unknown
            reply += reply_error
            i += 1
    return reply


logging.basicConfig(format="%(asctime)s %(message)s", level=logging.INFO)
logger = logging.getLogger("pyindi-synscan")


# initialize INDI device
indiclient = IndiClient()
indiclient.setServer(INDI_SERVER_HOST, INDI_SERVER_PORT)
indiclient.watchDevice(TELESCOPE_DEVICE)
device = None

# Connect server and device before launching serial port listening
# timeout is 2 secs in tty_read from the synscan driver
logger.info(
    "Connecting server " + indiclient.getHost() + ":" + str(indiclient.getPort())
)
serverconnected = indiclient.connectServer()
while not (serverconnected):
    logger.info(
        "No indiserver running on "
        + indiclient.getHost()
        + ":"
        + str(indiclient.getPort())
    )
    time.sleep(2)
    serverconnected = indiclient.connectServer()
if not (device):
    device = indiclient.getDevice(TELESCOPE_DEVICE)
    while not (device):
        logger.info("Trying to get device " + TELESCOPE_DEVICE)
        time.sleep(0.5)
        device = indiclient.getDevice(TELESCOPE_DEVICE)
logger.info("Got device " + TELESCOPE_DEVICE)

if not (device.isConnected()):
    if TELESCOPE_SIMULATION:
        logger.info("setting " + TELESCOPE_SIMPROP + " On")
        device_sim = device.getSwitch(TELESCOPE_SIMPROP)
        while not (device_sim):
            logger.info("Trying to get poperty " + TELESCOPE_SIMPROP)
            time.sleep(0.5)
            device_sim = device.getSwitch(TELESCOPE_SIMPROP)

        device_sim[0].setState(PyIndi.ISS_ON)  # the "ENABLE" switch
        device_sim[1].setState(PyIndi.ISS_OFF)  # the "DISABLE" switch
        indiclient.sendNewProperty(device_sim)

if not (device.isConnected()):
    device_connect = device.getSwitch("CONNECTION")
    while not (device_connect):
        logger.info("Trying to connect device " + TELESCOPE_DEVICE)
        time.sleep(0.5)
        device_connect = device.getSwitch("CONNECTION")
if not (device.isConnected()):
    device_connect[0].setState(PyIndi.ISS_ON)  # the "CONNECT" switch
    device_connect[1].setState(PyIndi.ISS_OFF)  # the "DISCONNECT" switch
    indiclient.sendNewProperty(device_connect)
while not (device.isConnected()):
    time.sleep(0.2)
logger.info("Device " + TELESCOPE_DEVICE + " connected")

if USE_SERIAL:
    logger.info("Creating virtual serial port " + DEVICE_PORT)
    # create the virtual serial port
    master, slave = pty.openpty()
    # and link it to /tmp/indi-synscan
    os.symlink(os.ttyname(slave), DEVICE_PORT)

if USE_TCP:
    # Create a TCP/IP socket
    server_name = "0.0.0.0"
    server_port = TCP_LISTEN_PORT
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_address = (server_name, server_port)
    logger.info(
        "Starting up on {0} port {1}\n".format(server_address[0], server_address[1])
    )
    sock.bind(server_address)
    sock.listen(1)

rlist = [master, sock]
wlist = []
xlist = [master, sock]

try:
    while True:
        rready, wready, eready = select.select(
            rlist, wlist, xlist, 1
        )  # timeout 1 sec to catch ctrl-C
        for f in rready:
            if f == master:
                # serial
                chars = os.read(master, 1024)
                if sys.version_info < (3,):
                    chars = bytearray(chars)
                # while chars!=b'':
                logger.info("read: " + repr(chars))
                reply = process_command(chars, indiclient, logger)
                # logger.info("write: "+ repr(reply))
                for c in reply.split(b"#")[:-1]:
                    logger.info("write: " + repr(c + b"#"))
                    os.write(master, c + b"#")
                    # chars=os.read(master, 1024)
                    # if sys.version_info < (3,):
                    #    chars=bytearray(chars)
            if f == sock:
                # SkySafari v4.0.1 continously opens and closed the connection,
                # while Stellarium via socat opens it and keeps it open using:
                # $ ./socat GOPEN:/dev/ptyp0,ignoreeof TCP:raspberrypi8:4030
                # (probably socat which is maintaining the link)
                # sys.stdout.write("waiting for a connection\n")
                connection, client_address = sock.accept()
                chars = ""
                try:
                    logger.info(
                        "Client connected: {0}, {1}".format(
                            client_address[0], client_address[1]
                        )
                    )
                    while True:
                        chars = connection.recv(1024)
                        if len(chars) == 0:
                            break
                        if sys.version_info < (3,):
                            chars = bytearray(chars)
                        logger.info("read: " + repr(chars))
                        reply = process_command(chars, indiclient, logger)
                        if reply != b"":
                            connection.sendall(reply)
                            logger.info("write: " + repr(reply))
                        else:
                            logger.info("nothing to respond")
                finally:
                    connection.close()

except KeyboardInterrupt:
    logger.info("pyindi-synscan stopped (Ctrl-C)")

if USE_SERIAL:
    os.remove(DEVICE_PORT)



import sys
import os
import glob

# Find the build directory dynamically
build_dir = os.path.abspath(
    os.path.join(os.path.dirname(os.path.dirname(__file__)), "build")
)
lib_dirs = glob.glob(os.path.join(build_dir, "lib.*"))
if lib_dirs:
    # Add the first lib directory to the Python path
    sys.path.insert(0, lib_dirs[0])
else:
    # If no lib directory is found, try to find the wheel file
    wheel_files = glob.glob(
        os.path.join(os.path.dirname(os.path.dirname(__file__)), "dist", "*.whl")
    )
    if wheel_files:
        # Extract the wheel file to a temporary directory
        import tempfile
        import zipfile

        temp_dir = tempfile.mkdtemp()
        with zipfile.ZipFile(wheel_files[0], "r") as zip_ref:
            zip_ref.extractall(temp_dir)
        sys.path.insert(0, temp_dir)



# for logging
import sys
import time
import logging

# import the PyIndi module
import PyIndi


# The IndiClient class which inherits from the module PyIndi.BaseClient class
# Note that all INDI constants are accessible from the module as PyIndi.CONSTANTNAME
class IndiClient(PyIndi.BaseClient):
    def __init__(self):
        super(IndiClient, self).__init__()
        self.logger = logging.getLogger("IndiClient")
        self.logger.info("creating an instance of IndiClient")

    def newDevice(self, d):
        """Emmited when a new device is created from INDI server."""
        self.logger.info(f"new device {d.getDeviceName()}")

    def removeDevice(self, d):
        """Emmited when a device is deleted from INDI server."""
        self.logger.info(f"remove device {d.getDeviceName()}")

    def newProperty(self, p):
        """Emmited when a new property is created for an INDI driver."""
        self.logger.info(
            f"new property {p.getName()} as {p.getTypeAsString()} for device {p.getDeviceName()}"
        )

    def updateProperty(self, p):
        """Emmited when a new property value arrives from INDI server."""
        self.logger.info(
            f"update property {p.getName()} as {p.getTypeAsString()} for device {p.getDeviceName()}"
        )

    def removeProperty(self, p):
        """Emmited when a property is deleted for an INDI driver."""
        self.logger.info(
            f"remove property {p.getName()} as {p.getTypeAsString()} for device {p.getDeviceName()}"
        )

    def newMessage(self, d, m):
        """Emmited when a new message arrives from INDI server."""
        self.logger.info(f"new Message {d.messageQueue(m)}")

    def serverConnected(self):
        """Emmited when the server is connected."""
        self.logger.info(f"Server connected ({self.getHost()}:{self.getPort()})")

    def serverDisconnected(self, code):
        """Emmited when the server gets disconnected."""
        self.logger.info(
            f"Server disconnected (exit code = {code},{self.getHost()}:{self.getPort()})"
        )


logging.basicConfig(format="%(asctime)s %(message)s", level=logging.INFO)

# Create an instance of the IndiClient class and initialize its host/port members
indiClient = IndiClient()
indiClient.setServer("localhost", 7624)

# Connect to server
print("Connecting and waiting 1 sec")
if not indiClient.connectServer():
    print(
        f"No indiserver running on {indiClient.getHost()}:{indiClient.getPort()} - Try to run"
    )
    print("  indiserver indi_simulator_telescope indi_simulator_ccd")
    sys.exit(1)

# Waiting for discover devices
time.sleep(1)

# Print list of devices. The list is obtained from the wrapper function getDevices as indiClient is an instance
# of PyIndi.BaseClient and the original C++ array is mapped to a Python List. Each device in this list is an
# instance of PyIndi.BaseDevice, so we use getDeviceName to print its actual name.
print("List of devices")
deviceList = indiClient.getDevices()
for device in deviceList:
    print(f"   > {device.getDeviceName()}")

# Print all properties and their associated values.
print("List of Device Properties")
for device in deviceList:
    print(f"-- {device.getDeviceName()}")
    genericPropertyList = device.getProperties()

    for genericProperty in genericPropertyList:
        print(f"   > {genericProperty.getName()} {genericProperty.getTypeAsString()}")

        if genericProperty.getType() == PyIndi.INDI_TEXT:
            for widget in PyIndi.PropertyText(genericProperty):
                print(
                    f"       {widget.getName()}({widget.getLabel()}) = {widget.getText()}"
                )

        if genericProperty.getType() == PyIndi.INDI_NUMBER:
            for widget in PyIndi.PropertyNumber(genericProperty):
                print(
                    f"       {widget.getName()}({widget.getLabel()}) = {widget.getValue()}"
                )

        if genericProperty.getType() == PyIndi.INDI_SWITCH:
            for widget in PyIndi.PropertySwitch(genericProperty):
                print(
                    f"       {widget.getName()}({widget.getLabel()}) = {widget.getStateAsString()}"
                )

        if genericProperty.getType() == PyIndi.INDI_LIGHT:
            for widget in PyIndi.PropertyLight(genericProperty):
                print(
                    f"       {widget.getLabel()}({widget.getLabel()}) = {widget.getStateAsString()}"
                )

        if genericProperty.getType() == PyIndi.INDI_BLOB:
            for widget in PyIndi.PropertyBlob(genericProperty):
                print(
                    f"       {widget.getName()}({widget.getLabel()}) = <blob {widget.getSize()} bytes>"
                )

# Disconnect from the indiserver
print("Disconnecting")
indiClient.disconnectServer()



import PyIndi
import time
import sys
import threading


class IndiClient(PyIndi.BaseClient):
    def __init__(self):
        super(IndiClient, self).__init__()

    def updateProperty(self, prop):
        global blobEvent
        if prop.getType() == PyIndi.INDI_BLOB:
            print("new BLOB ", prop.getName())
            blobEvent.set()


# connect the server
indiclient = IndiClient()
indiclient.setServer("localhost", 7624)

if not indiclient.connectServer():
    print(
        "No indiserver running on "
        + indiclient.getHost()
        + ":"
        + str(indiclient.getPort())
        + " - Try to run"
    )
    print("  indiserver indi_simulator_telescope indi_simulator_ccd")
    sys.exit(1)

# connect the scope
telescope = "Telescope Simulator"
device_telescope = None
telescope_connect = None

# get the telescope device
device_telescope = indiclient.getDevice(telescope)
while not device_telescope:
    time.sleep(0.5)
    device_telescope = indiclient.getDevice(telescope)

# wait CONNECTION property be defined for telescope
telescope_connect = device_telescope.getSwitch("CONNECTION")
while not telescope_connect:
    time.sleep(0.5)
    telescope_connect = device_telescope.getSwitch("CONNECTION")

# if the telescope device is not connected, we do connect it
if not device_telescope.isConnected():
    # Property vectors are mapped to iterable Python objects
    # Hence we can access each element of the vector using Python indexing
    # each element of the "CONNECTION" vector is a ISwitch
    telescope_connect.reset()
    telescope_connect[0].setState(PyIndi.ISS_ON)  # the "CONNECT" switch
    indiclient.sendNewSwitch(telescope_connect)  # send this new value to the device

# Now let's make a goto to vega
# Beware that ra/dec are in decimal hours/degrees
vega = {"ra": (279.23473479 * 24.0) / 360.0, "dec": +38.78368896}

# We want to set the ON_COORD_SET switch to engage tracking after goto
# device.getSwitch is a helper to retrieve a property vector
telescope_on_coord_set = device_telescope.getSwitch("ON_COORD_SET")
while not telescope_on_coord_set:
    time.sleep(0.5)
    telescope_on_coord_set = device_telescope.getSwitch("ON_COORD_SET")
# the order below is defined in the property vector, look at the standard Properties page
# or enumerate them in the Python shell when you're developing your program
telescope_on_coord_set.reset()
telescope_on_coord_set[0].setState(PyIndi.ISS_ON)  # index 0-TRACK, 1-SLEW, 2-SYNC
indiclient.sendNewSwitch(telescope_on_coord_set)

# We set the desired coordinates
telescope_radec = device_telescope.getNumber("EQUATORIAL_EOD_COORD")
while not telescope_radec:
    time.sleep(0.5)
    telescope_radec = device_telescope.getNumber("EQUATORIAL_EOD_COORD")
telescope_radec[0].setValue(vega["ra"])
telescope_radec[1].setValue(vega["dec"])
indiclient.sendNewNumber(telescope_radec)

# and wait for the scope has finished moving
while telescope_radec.getState() == PyIndi.IPS_BUSY:
    print("Scope Moving ", telescope_radec[0].value, telescope_radec[1].value)
    time.sleep(2)

# Let's take some pictures
ccd = "CCD Simulator"
device_ccd = indiclient.getDevice(ccd)
while not (device_ccd):
    time.sleep(0.5)
    device_ccd = indiclient.getDevice(ccd)

ccd_connect = device_ccd.getSwitch("CONNECTION")
while not (ccd_connect):
    time.sleep(0.5)
    ccd_connect = device_ccd.getSwitch("CONNECTION")
if not (device_ccd.isConnected()):
    ccd_connect.reset()
    ccd_connect[0].setState(PyIndi.ISS_ON)  # the "CONNECT" switch
    indiclient.sendNewSwitch(ccd_connect)

ccd_exposure = device_ccd.getNumber("CCD_EXPOSURE")
while not (ccd_exposure):
    time.sleep(0.5)
    ccd_exposure = device_ccd.getNumber("CCD_EXPOSURE")

# Ensure the CCD simulator snoops the telescope simulator
# otherwise you may not have a picture of vega
ccd_active_devices = device_ccd.getText("ACTIVE_DEVICES")
while not (ccd_active_devices):
    time.sleep(0.5)
    ccd_active_devices = device_ccd.getText("ACTIVE_DEVICES")
ccd_active_devices[0].setText("Telescope Simulator")
indiclient.sendNewText(ccd_active_devices)

# we should inform the indi server that we want to receive the
# "CCD1" blob from this device
indiclient.setBLOBMode(PyIndi.B_ALSO, ccd, "CCD1")

ccd_ccd1 = device_ccd.getBLOB("CCD1")
while not ccd_ccd1:
    time.sleep(0.5)
    ccd_ccd1 = device_ccd.getBLOB("CCD1")

# a list of our exposure times
exposures = [1.0, 5.0]

# we use here the threading.Event facility of Python
# we define an event for newBlob event
blobEvent = threading.Event()
blobEvent.clear()
i = 0
ccd_exposure[0].setValue(exposures[i])
indiclient.sendNewNumber(ccd_exposure)
while i < len(exposures):
    # wait for the ith exposure
    blobEvent.wait()
    # we can start immediately the next one
    if i + 1 < len(exposures):
        ccd_exposure[0].setValue(exposures[i + 1])
        blobEvent.clear()
        indiclient.sendNewNumber(ccd_exposure)
    # and meanwhile process the received one
    for blob in ccd_ccd1:
        print(
            "name: ",
            blob.getName(),
            " size: ",
            blob.getSize(),
            " format: ",
            blob.getFormat(),
        )
        # pyindi-client adds a getblobdata() method to IBLOB item
        # for accessing the contents of the blob, which is a bytearray in Python
        fits = blob.getblobdata()
        print("fits data type: ", type(fits))
        # here you may use astropy.io.fits to access the fits data
    # and perform some computations while the ccd is exposing
    # but this is outside the scope of this tutorial
    i += 1



import PyIndi
import logging
import pytest
import time


logging.basicConfig(level=logging.INFO)
logger = logging


class IndiClient(PyIndi.BaseClient):
    def __init__(self):
        super(IndiClient, self).__init__()

        self._blob_data = None

    @property
    def blob_data(self):
        return self._blob_data

    def updateProperty(self, p):
        # INDI 2.x.x code path

        if p.getType() == PyIndi.INDI_BLOB:
            p_blob = PyIndi.PropertyBlob(p)
            self.processBlob(p_blob[0])
        elif p.getType() == PyIndi.INDI_NUMBER:
            pass
        elif p.getType() == PyIndi.INDI_SWITCH:
            pass
        elif p.getType() == PyIndi.INDI_TEXT:
            pass
        elif p.getType() == PyIndi.INDI_LIGHT:
            pass
        else:
            logger.warning("Property type not matched: %d", p.getType())

    def processBlob(self, blob):
        self._blob_data = blob.getblobdata()


@pytest.fixture()
def client():
    client = IndiClient()
    client.setServer("localhost", 7624)
    client.connectServer()

    for _ in range(15):
        if client.isServerConnected() and len(client.getDevices()) > 0:
            break
        time.sleep(1)

    yield client


def test_connect(client):
    assert client.isServerConnected()


def test_list_devices(client):
    device_names = [x.getDeviceName() for x in client.getDevices()]
    expected_device_names = [
        "Telescope Simulator",
        "Filter Simulator",
        "Guide Simulator",
        "GPS Simulator",
        "Focuser Simulator",
        "CCD Simulator",
    ]
    assert set(device_names) == set(expected_device_names)


def test_connecting_devices(client):
    for device in client.getDevices():
        if not device.isConnected():
            connection = device.getSwitch("CONNECTION")
            time.sleep(1.0)

            connection[0].setState(PyIndi.ISS_ON)  # CONNECT
            connection[1].setState(PyIndi.ISS_OFF)  # DISCONNECT
            client.sendNewSwitch(connection)

            time.sleep(1.0)

        assert device.isConnected()


def test_getting_properties(client):
    for device in client.getDevices():
        if not device.isConnected():
            connection = device.getSwitch("CONNECTION")
            time.sleep(1.0)

            connection[0].setState(PyIndi.ISS_ON)  # CONNECT
            connection[1].setState(PyIndi.ISS_OFF)  # DISCONNECT
            client.sendNewSwitch(connection)

            time.sleep(1.0)

        for prop in device.getProperties():
            prop_name = prop.getName()
            prop_type = prop.getType()
            if prop_type == PyIndi.INDI_TEXT:
                prop_text = prop.getText()
            elif prop_type == PyIndi.INDI_NUMBER:
                prop_number = prop.getNumber()
            elif prop_type == PyIndi.INDI_SWITCH:
                prop_switch = prop.getSwitch()
            elif prop_type == PyIndi.INDI_LIGHT:
                prop_light = prop.getLight()
            elif prop_type == PyIndi.INDI_BLOB:
                prop_blob = prop.getBLOB()


def test_exposure(client):
    device_ccd = client.getDevice("CCD Simulator")
    time.sleep(1.0)

    connection = device_ccd.getSwitch("CONNECTION")
    time.sleep(1.0)

    if not device_ccd.isConnected():
        connection[0].setState(PyIndi.IIS_ON)  # CONNECT
        connection[1].setState(PyIndi.ISS_OFF)  # DISCONNECT
        client.sendNewSwitch(connection)

        time.sleep(1.0)

    client.setBLOBMode(PyIndi.B_ALSO, device_ccd.getDeviceName(), None)

    EXPOSURE = device_ccd.getNumber("CCD_EXPOSURE")
    time.sleep(1.0)

    # take 0.1 second exposure
    EXPOSURE[0].setValue(0.1)  # CCD_EXPOSURE_VALUE
    client.sendNewNumber(EXPOSURE)

    time.sleep(3.0)

    assert not isinstance(client.blob_data, type(None))


def test_slew(client):
    device_telescope = client.getDevice("Telescope Simulator")
    time.sleep(1)

    if not device_telescope.isConnected():
        connection = device_telescope.getSwitch("CONNECTION")
        time.sleep(1)

        connection[0].setState(PyIndi.ISS_ON)  # CONNECT
        connection[1].setState(PyIndi.ISS_OFF)  # DISCONNECT
        client.sendNewSwitch(connection)

        time.sleep(1.0)

    device_ccd = client.getDevice("CCD Simulator")
    time.sleep(1)

    if not device_ccd.isConnected():
        connection = device_ccd.getSwitch("CONNECTION")
        time.sleep(1)

        connection[0].setState(PyIndi.ISS_ON)  # CONNECT
        connection[1].setState(PyIndi.ISS_OFF)  # DISCONNECT
        client.sendNewSwitch(connection)

        time.sleep(1.0)

    ON_COORD_SET = device_telescope.getSwitch("ON_COORD_SET")
    time.sleep(1.0)

    ON_COORD_SET[0].setState(PyIndi.ISS_ON)  # TRACK
    ON_COORD_SET[1].setState(PyIndi.ISS_OFF)  # SLEW
    ON_COORD_SET[2].setState(PyIndi.ISS_OFF)  # SYNC
    client.sendNewSwitch(ON_COORD_SET)

    EQUATORIAL_EOD_COORD = device_telescope.getNumber("EQUATORIAL_EOD_COORD")
    time.sleep(1.0)

    # slew to M13
    EQUATORIAL_EOD_COORD[0].setValue(16.7175)  # RA_PE
    EQUATORIAL_EOD_COORD[1].setValue(36.4233)  # DEC_PE
    client.sendNewNumber(EQUATORIAL_EOD_COORD)



# A python server for the stellarium planetarium program
# it implements the stellarium Telescope Control plugin protocol.
import signal, os, sys, logging, time, calendar, math, traceback
import socket, select

import PyIndi

indihost = "localhost"
indiport = 7624
inditelescope = "Telescope Simulator"
indiclient = None
autoconnect = True
indiServerConnected = False
isIndiTelescopeConnected = False
indiTelescopeRAJNOW = 0.0
indiTelescopeDECJNOW = 0.0
indiTelescopeTIMEUTC = ""
gotoQueue = []

stelport = 10001
stelSocket = None
# current stellarium clients
stelClients = {}

killed = False


def to_be(n, size):
    b = bytearray(size)
    i = size - 1
    while i >= 0:
        b[i] = n % 256
        n = n >> 8
        i -= 1
    return b


def from_be(b):
    n = 0
    for i in range(len(b)):
        n = (n << 8) + b[i]
    return n


def to_le(n, size):
    b = bytearray(size)
    i = 0
    while i < size:
        b[i] = n % 256
        n = n >> 8
        i += 1
    return b


def from_le(b):
    n = 0
    for i in range(len(b) - 1, -1, -1):
        n = (n << 8) + b[i]
    return n


# Simple class to keep stellarium socket connections
class StelClient:
    def __init__(self, sock, clientaddress):
        self.socket = sock
        self.clientaddress = clientaddress
        self.writebuf = bytearray(120)
        self.readbuf = bytearray(120)
        self.recv = 0
        self.msgq = []
        self.tosend = 0

    def hasToWrite(self):
        return self.tosend > 0

    def performRead(self):
        # logging.info('Socket '+str(self.socket.fileno()) + ' has to read')
        buf = bytearray(120 - self.recv)
        nrecv = self.socket.recv_into(buf, 120 - self.recv)
        # logging.info('Socket '+str(self.socket.fileno()) + 'read: '+str(buf))
        if nrecv <= 0:
            logging.info("Client " + str(self.socket.fileno()) + " is away")
            self.disconnect()
            stelClients.pop(self.socket)
            return
        self.readbuf[self.recv : self.recv + nrecv] = buf
        self.recv += nrecv
        last = self.datareceived()
        if last > 0:
            self.readbuf = self.readbuf[last:]
            self.recv -= last

    def datareceived(self):
        global gotoQueue
        p = 0
        while p < self.recv - 2:
            psize = from_le(self.readbuf[p : p + 2])
            if psize > len(self.readbuf) - p:
                break
            ptype = from_le(self.readbuf[p + 2 : p + 4])
            if ptype == 0:
                micros = from_le(self.readbuf[p + 4 : p + 12])
                if abs((micros / 1000000.0) - int(time.time())) > 60.0:
                    logging.warning(
                        "Client "
                        + str(self.socket.fileno())
                        + " clock differs for more than one minute: "
                        + str(int(micros / 1000000.0))
                        + "/"
                        + str(int(time.time()))
                    )
                targetraint = from_le(self.readbuf[p + 12 : p + 16])
                targetdecint = from_le(self.readbuf[p + 16 : p + 20])
                if targetdecint > (4294967296 / 2):
                    targetdecint = -(4294967296 - targetdecint)
                targetra = (targetraint * 24.0) / 4294967296.0
                targetdec = (targetdecint * 360.0) / 4294967296.0
                logging.info(
                    "Queuing goto (ra, dec)=("
                    + str(targetra)
                    + ", "
                    + str(targetdec)
                    + ")"
                )
                gotoQueue.append((targetra, targetdec))
                p += psize
            else:
                p += psize
        return p

    def performWrite(self):
        global stelClients
        # logging.info('Socket '+str(self.socket.fileno()) + ' will write')
        sent = self.socket.send(self.writebuf[0 : self.tosend])
        if sent <= 0:
            logging.info("Client " + str(self.socket.fileno()) + " is away")
            self.disconnect()
            stelClients.pop(self.socket)
            return
        self.writebuf = self.writebuf[sent:]
        self.tosend -= sent
        if self.tosend == 0:
            if len(self.msgq) > 0:
                self.writebuf[0 : len(self.msgq[0])] = self.msgq[0]
                self.tosend = len(msgq[0])
                self.msgq = self.msgq[1:]

    def sendMsg(self, msg):
        if self.tosend == 0:
            self.writebuf[0 : len(msg)] = msg
            self.tosend = len(msg)
        else:
            self.msgq.append(msg)

    def sendEqCoords(self, utc, rajnow, decjnow, status):
        msg = bytearray(24)
        msg[0:2] = to_le(24, 2)
        msg[2:4] = to_le(0, 2)
        if utc != "":
            try:
                tstamp = calendar.timegm(time.strptime(utc, "%Y-%m-%dT%H:%M:%S"))
            except:
                tstamp = 0
        else:
            # Simulator does not send its UTC time, and timestamp are emptied somewhere
            tstamp = int(time.time())
        msg[4:12] = to_le(tstamp, 8)
        msg[12:16] = to_le(int(math.floor(rajnow * (4294967296.0 / 24.0))), 4)
        msg[16:20] = to_le(int(math.floor(decjnow * (4294967296.0 / (360.0)))), 4)
        msg[20:24] = to_le(status, 4)
        self.sendMsg(msg)

    def disconnect(self):
        try:
            self.socket.shutdown(socket.SHUT_RDWR)
            self.socket.close()
        except:
            traceback.print_exc()


# The IndiClient class which manages connections to the indi server
# and events from the telescope device.
# Keeps track (through python global variables) of the connection/deconnection
# and RA/DEC values of the telescope device.
# Beware that swig generally makes a copy of the C++ Object when calling a python method,
# so we must also make python-side copy of what we need.
class IndiClient(PyIndi.BaseClient):
    def __init__(self, telescope):
        super(IndiClient, self).__init__()
        self.logger = logging.getLogger("PyIndi.BaseClient")
        self.telescope = telescope
        self.tdevice = None

    # These new* and server* virtual methods live in a C++ thread.
    # Beware of their interaction with other python threads: swig locks
    # the python GIL before calling them from C++  and releases it at the end.
    # So it is safe to modify global python variables here.
    def newDevice(self, d):
        # self.logger.info("new device " + d.getDeviceName())
        if d.getDeviceName() != self.telescope:
            self.logger.info("Receiving " + d.getDeviceName() + " Device...")
        self.tdevice = d

    def newProperty(self, p):
        global autoconnect, isIndiTelescopeConnected, indiTelescopeRAJNOW, indiTelescopeDECJNOW
        # self.logger.info("new property "+ p.getName() + " for device "+ p.getDeviceName())
        if p.getDeviceName() == self.telescope:
            if p.isNameMatch("CONNECTION"):
                if not (self.tdevice.isConnected()) and autoconnect:
                    self.logger.info("Autoconnecting device " + p.getDeviceName())
                    self.connectDevice(self.telescope)
                if self.tdevice.isConnected():
                    self.logger.info("Found connected device " + p.getDeviceName())
                    isIndiTelescopeConnected = True
            if p.isNameMatch("EQUATORIAL_EOD_COORD"):
                nvp = PyIndi.PropertyNumber(p)
                indiTelescopeRAJNOW = nvp[0].getValue()
                indiTelescopeDECJNOW = nvp[1].getValue()
                self.logger.info(
                    "Got JNow Eq. coords for "
                    + p.getDeviceName()
                    + ": (ra, dec)=("
                    + str(indiTelescopeRAJNOW)
                    + ", "
                    + str(indiTelescopeDECJNOW)
                    + ")"
                )

    def updateProperty(self, p):
        if p.getDeviceName() != self.telescope:
            return

        global isIndiTelescopeConnected
        if p.isNameMatch("CONNECTION"):
            svp = PyIndi.PropertySwitch(p)
            if svp[0].getState() == PyIndi.ISS_ON:
                isIndiTelescopeConnected = True
            if svp[1].getState() == PyIndi.ISS_ON:
                isIndiTelescopeConnected = False

        global indiTelescopeRAJNOW, indiTelescopeDECJNOW
        if p.isNameMatch("EQUATORIAL_EOD_COORD"):
            nvp = PyIndi.PropertyNumber(p)
            indiTelescopeRAJNOW = nvp[0].getValue()
            indiTelescopeDECJNOW = nvp[1].getValue()
            # self.logger.info ("RA/DEC Timestamp "+str(nvp.getTimestamp()))

        global indiTelescopeTIMEUTC
        if p.isNameMatch("TIME_UTC"):
            tvp = PyIndi.PropertyText(p)
            indiTelescopeTIMEUTC = tvp[0].getText()
            self.logger.info("UTC Time " + str(tvp[0].getText()))

    def serverConnected(self):
        global indiServerConnected
        indiServerConnected = True
        self.logger.info(
            "Server connected (" + self.getHost() + ":" + str(self.getPort()) + ")"
        )

    def serverDisconnected(self, code):
        global indiServerConnected
        self.logger.info(
            "Server disconnected (exit code = "
            + str(code)
            + ","
            + str(self.getHost())
            + ":"
            + str(self.getPort())
            + ")"
        )
        indiServerConnected = False

    # You may extend the BaseClient class with your own python methods.
    # These ones will live in the main python thread.
    def waitServer(self):
        while not self.connectServer():
            self.logger.info(
                "No indiserver running on " + self.getHost() + ":" + str(self.getPort())
            )
            time.sleep(5)


def terminate(signum, frame):
    killed = True


# how to get back this signal which is translated in a python exception ?
# signal.signal(signal.SIGKILL, terminate)
signal.signal(signal.SIGHUP, terminate)
signal.signal(signal.SIGQUIT, terminate)

logging.basicConfig(format="%(asctime)s %(message)s", level=logging.INFO)

# Create an instance of the IndiClient class and initialize its host/port members
indiclient = IndiClient(inditelescope)
indiclient.setServer(indihost, indiport)

# Whereas connection to the indiserver will be handled by the C++ thread and the
# above callbacks, connection from the stellarium client programs will be managed
# in the main python thread: we use the usual select method with non-blocking sockets,
# listening on the stellarium port (10001) and using buffered reads/writes on the
# connected stellarium client sockets.
stelSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
stelSocket.bind(("", stelport))
stelSocket.listen(5)
stelSocket.setblocking(0)
stelSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
status = 0
try:
    while not killed:
        # try to reconnect indi server if server restarted
        if not (indiServerConnected):
            # Connect to the indi server restricting new* messages to our telescope device
            indiclient.watchDevice(inditelescope)
            indiclient.waitServer()
            logging.info(
                "Connected to indiserver@"
                + indiclient.getHost()
                + ":"
                + str(indiclient.getPort())
                + ', watching "'
                + inditelescope
                + '" device'
            )
        if isIndiTelescopeConnected:
            # logging.info('RA='+str(indiTelescopeRAJNOW)+', DEC='+str(indiTelescopeDECJNOW))
            for s in stelClients:
                stelClients[s].sendEqCoords(
                    indiTelescopeTIMEUTC,
                    indiTelescopeRAJNOW,
                    indiTelescopeDECJNOW,
                    status,
                )
            if len(gotoQueue) > 0:
                logging.info("Sending goto (ra, dec)=" + str(gotoQueue[0]))
                d = indiclient.getDevice(inditelescope)
                oncoordset = d.getSwitch("ON_COORD_SET")
                oncoordset.reset()
                oncoordset[0].setState(PyIndi.ISS_ON)
                indiclient.sendNewProperty(oncoordset)

                eqeodcoords = d.getNumber("EQUATORIAL_EOD_COORD")
                eqeodcoords[0].setValue(gotoQueue[0][0])
                eqeodcoords[1].setValue(gotoQueue[0][1])
                gotoQueue = gotoQueue[1:]
                indiclient.sendNewProperty(eqeodcoords)
        # logging.info('Perform step')
        # perform one step
        readers = [stelSocket] + [s for s in stelClients]
        writers = [s for s in stelClients if stelClients[s].hasToWrite()]
        ready_to_read, ready_to_write, in_error = select.select(
            readers, writers, [], 0.5
        )
        for r in ready_to_read:
            if r == stelSocket:
                news, newa = stelSocket.accept()
                news.setblocking(0)
                stelClients[news] = StelClient(news, newa)
                logging.info(
                    "New Stellarium client "
                    + str(news.fileno())
                    + " on port "
                    + str(newa)
                )
            else:
                stelClients[r].performRead()
        for r in ready_to_write:
            if r in stelClients.keys():
                stelClients[r].performWrite()
        for r in in_error:
            logging.info("Lost Stellarium client " + str(r.fileno()))
            if r in stelClients.keys():
                stelClients[r].disconnect()
                stelClients.pop(r)
        time.sleep(0.5)
except KeyboardInterrupt:
    logging.info("Bye")
else:
    traceback.print_exc()

stelSocket.shutdown(socket.SHUT_RDWR)
stelSocket.close()
for sc in stelClients:
    stelClients[sc].disconnect()
stelSocket.close()
indiclient.disconnectServer()

sys.exit(0)



import PyIndi


class IndiClient(PyIndi.BaseClient):
    def __init__(self):
        super(IndiClient, self).__init__()


indiclient = IndiClient()
indiclient.setServer("localhost", 7624)

indiclient.connectServer()
while 1:
    pass



from setuptools import setup, Extension, Command
import os
import sys
import subprocess
from configparser import ConfigParser
from setuptools.command.build_ext import build_ext as _build_ext

# Default values in case setup.cfg is not available
DEFAULT_INCLUDE_DIRS = ["/usr/include", "/usr/include/libindi", "/usr/local/include/libindi"]
DEFAULT_LIBRARIES = ["z", "cfitsio", "nova", "indiclient"]
DEFAULT_SWIG_OPTS = ["-v", "-Wall", "-c++", "-threads", "-I/usr/include", "-I/usr/include/libindi", "-I/usr/local/include/libindi"]

# Try to load build configurations from setup.cfg
try:
    config = ConfigParser()
    config.read("setup.cfg")
    
    if "build_ext" in config:
        include_dirs = config.get("build_ext", "include_dirs", fallback=":".join(DEFAULT_INCLUDE_DIRS)).split(":")
        libraries = config.get("build_ext", "libraries", fallback=" ".join(DEFAULT_LIBRARIES)).split()
        swig_opts_str = config.get("build_ext", "swig_opts", fallback=" ".join(DEFAULT_SWIG_OPTS))
        # Parse swig_opts, handling quotes properly
        swig_opts = []
        current_opt = ""
        in_quotes = False
        for char in swig_opts_str:
            if char == '"' or char == "'":
                in_quotes = not in_quotes
            elif char == ' ' and not in_quotes:
                if current_opt:
                    swig_opts.append(current_opt)
                    current_opt = ""
            else:
                current_opt += char
        if current_opt:
            swig_opts.append(current_opt)
    else:
        include_dirs = DEFAULT_INCLUDE_DIRS
        libraries = DEFAULT_LIBRARIES
        swig_opts = DEFAULT_SWIG_OPTS
except Exception as e:
    print(f"Warning: Error reading setup.cfg: {e}. Using default values.")
    include_dirs = DEFAULT_INCLUDE_DIRS
    libraries = DEFAULT_LIBRARIES
    swig_opts = DEFAULT_SWIG_OPTS

# Custom command to run SWIG
class SwigCommand(Command):
    description = "Run SWIG to generate Python bindings"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # Find indiclientpython.i
        swig_file = "indiclientpython.i"
        search_paths = [".", "/pyindi-client"]
        
        swig_file_path = None
        for path in search_paths:
            full_path = os.path.join(path, swig_file)
            if os.path.exists(full_path):
                swig_file_path = full_path
                break
        
        if not swig_file_path:
            print(f"Error: {swig_file} not found in any of the search paths: {search_paths}")
            print(f"Current directory: {os.getcwd()}")
            print(f"Directory contents: {os.listdir('.')}")
            sys.exit(1)
            
        # Run SWIG command
        # Add -outdir PyIndi to place the generated Python module in the correct directory
        cmd = ["swig", "-python", "-c++", "-threads", "-outdir", "PyIndi"] + swig_opts + [swig_file_path]
        print(f"Running SWIG command: {' '.join(cmd)}")
        try:
            subprocess.check_call(cmd)
            print("SWIG command completed successfully")
            
            # Check if the wrapper file was generated
            wrapper_file = "indiclientpython_wrap.cxx" # Should still be generated in root
            python_module = "PyIndi/PyIndi.py" # Check in the output directory
            
            # Check for C++ wrapper
            if not os.path.exists(wrapper_file):
                print(f"Error: {wrapper_file} was not generated by SWIG")
                # Try to find it in other locations
                for path in [".", "/pyindi-client"]:
                    full_path = os.path.join(path, wrapper_file)
                    if os.path.exists(full_path):
                        print(f"Found {wrapper_file} at {full_path}")
                        # Copy it to the current directory
                        if path != ".":
                            import shutil
                            shutil.copy(full_path, wrapper_file)
                            print(f"Copied {full_path} to {wrapper_file}")
                        break
                else:
                    print(f"Could not find {wrapper_file} in any location")
                    sys.exit(1)
            
            # Check for Python module in the specified output directory
            if not os.path.exists(python_module):
                print(f"Error: {python_module} was not generated by SWIG in the PyIndi directory")
                # Optional: Add search logic here if needed, but primary check is the target dir
                sys.exit(1) # Exit if not found in the target dir

        except subprocess.CalledProcessError as e:
            print(f"Error running SWIG command: {e}")
            sys.exit(1)
        except FileNotFoundError:
            print("Error: SWIG not found. Please install SWIG.")
            sys.exit(1)

# Custom build_ext command that runs SWIG first
class BuildExt(_build_ext):
    def run(self):
        try:
            # Run SWIG first
            self.run_command('swig')
            # Then run the regular build_ext command
            _build_ext.run(self)
        except Exception as e:
            print(f"Error during build_ext: {e}")
            raise

# INDI Client Extension
ext_module = Extension(
    name="PyIndi._PyIndi",
    sources=["indiclientpython_wrap.cxx"],
    include_dirs=include_dirs,
    libraries=libraries,
    library_dirs=["/usr/lib", "/usr/lib64", "/lib", "/lib64"],
    extra_compile_args=["-fPIC"],
    extra_link_args=["-shared"],
)

setup(
    name="pyindi-client",
    version="2.1.7",  # Match version in pyproject.toml
    zip_safe=False,
    ext_modules=[ext_module],
    packages=["PyIndi"],
    package_data={
        "PyIndi": ["PyIndi.py"],
    },
    cmdclass={
        'swig': SwigCommand,
        'build_ext': BuildExt,
    },
)



# This file makes Python treat the directory as a package.

# Import contents of the PyIndi module into the package namespace
# for backward compatibility (allows 'import PyIndi')
from .PyIndi import *



import PyIndi
import time


class IndiClient(PyIndi.BaseClient):
    def __init__(self):
        super(IndiClient, self).__init__()

    def newDevice(self, d):
        global dmonitor
        # We catch the monitored device
        dmonitor = d
        print("New device ", d.getDeviceName())

    def newProperty(self, p):
        global monitored
        global cmonitor
        # we catch the "CONNECTION" property of the monitored device
        if p.getDeviceName() == monitored and p.isNameMatch("CONNECTION"):
            cmonitor = PyIndi.PropertySwitch(p)
        print("New property ", p.getName(), " for device ", p.getDeviceName())

    def updateProperty(self, p):
        global newval
        global prop
        nvp = PyIndi.PropertyNumber(p)
        if nvp.isValid():
            # We only monitor Number properties of the monitored device
            prop = nvp
            newval = True


monitored = "Telescope Simulator"
dmonitor = None
cmonitor = None

indiclient = IndiClient()
indiclient.setServer("localhost", 7624)

# we are only interested in the telescope device properties
indiclient.watchDevice(monitored)
indiclient.connectServer()

# wait CONNECTION property be defined
while not (cmonitor):
    time.sleep(0.5)

# if the monitored device is not connected, we do connect it
if not (dmonitor.isConnected()):
    # Property vectors are mapped to iterable Python objects
    # Hence we can access each element of the vector using Python indexing
    # each element of the "CONNECTION" vector is a ISwitch
    cmonitor[0].setState(PyIndi.ISS_ON)  # the "CONNECT" switch
    cmonitor[1].setState(PyIndi.ISS_OFF)  # the "DISCONNECT" switch
    indiclient.sendNewProperty(cmonitor)  # send this new value to the device

newval = False
prop = None
nrecv = 0
while nrecv < 10:
    # we poll the newval global variable
    if newval:
        print(
            "newval for property", prop.getName(), " of device ", prop.getDeviceName()
        )
        # prop is a property vector, mapped to an iterable Python object
        for n in prop:
            # n is a INumber as we only monitor number vectors
            print(n.getName(), " = ", n.getValue())
        nrecv += 1
        newval = False
